@@ -36,7 +36,7 @@
 
 /************************************************************************/
 
-#ifdef __UNIXOS2__
+#ifndef HAVE_SETEUID
 # define seteuid setuid
 #endif
 #define	iscomment(ch)	((ch) == '#' || (ch) == '\0')
@@ -46,37 +46,11 @@
 #define isreadable(f)	((access((f), R_OK) != -1) ? 1 : 0)
 #endif
 
-#ifndef __UNIXOS2__
 #define LC_PATHDELIM ':'
-#else
-#define LC_PATHDELIM ';'
-#endif
 
 #define XLC_BUFSIZE 256
 
-#ifndef X_NOT_POSIX
-#ifdef _POSIX_SOURCE
-#include <limits.h>
-#else
-#define _POSIX_SOURCE
-#include <limits.h>
-#undef _POSIX_SOURCE
-#endif
-#endif
-#ifndef PATH_MAX
-#ifdef WIN32
-#define PATH_MAX 512
-#else
-#include <sys/param.h>
-#endif
-#ifndef PATH_MAX
-#ifdef MAXPATHLEN
-#define PATH_MAX MAXPATHLEN
-#else
-#define PATH_MAX 1024
-#endif
-#endif
-#endif
+#include "pathmax.h"
 
 #define NUM_LOCALEDIR	64
 
@@ -114,75 +88,6 @@ parse_line(
     return argc;
 }
 
-#ifdef __UNIXOS2__
-
-/* fg021216: entries in locale files are separated by colons while under
-   OS/2, path entries are separated by semicolon, so we need two functions */
-
-static int
-parse_line1(
-    char *line,
-    char **argv,
-    int argsize)
-{
-    int argc = 0;
-    char *p = line;
-
-    while (argc < argsize) {
-	while (isspace(*p)) {
-	    ++p;
-	}
-	if (*p == '\0') {
-	    break;
-	}
-	argv[argc++] = p;
-	while (*p != ';' && *p != '\n' && *p != '\0') {
-	    ++p;
-	}
-	if (*p == '\0') {
-	    break;
-	}
-	*p++ = '\0';
-    }
-
-    return argc;
-}
-#elif defined(WIN32)
-
-/* this is parse_line but skips drive letters at the beginning of the entry */
-static int
-parse_line1(
-    char *line,
-    char **argv,
-    int argsize)
-{
-    int argc = 0;
-    char *p = line;
-
-    while (argc < argsize) {
-	while (isspace(*p)) {
-	    ++p;
-	}
-	if (*p == '\0') {
-	    break;
-	}
-	argv[argc++] = p;
-        if (isalpha(*p) && p[1] == ':') {
-            p+= 2; /* skip drive letters */
-        }
-	while (*p != ':' && *p != '\n' && *p != '\0') {
-	    ++p;
-	}
-	if (*p == '\0') {
-	    break;
-	}
-	*p++ = '\0';
-    }
-
-    return argc;
-}
-
-#endif   /* __UNIXOS2__ */
 
 /* Splits a colon separated list of directories, and returns the constituent
    paths (without trailing slash). At most argsize constituents are stored
@@ -196,7 +101,7 @@ _XlcParsePath(
     char *p = path;
     int n, i;
 
-#if !defined(__UNIXOS2__) && !defined(WIN32)
+#ifndef WIN32
     n = parse_line(path, argv, argsize);
 #else
     n = parse_line1(path, argv, argsize);
@@ -204,7 +109,7 @@ _XlcParsePath(
     for (i = 0; i < n; ++i) {
 	int len;
 	p = argv[i];
-	len = strlen(p);
+	len = (int) strlen(p);
 	if (len > 0 && p[len - 1] == '/') {
 	    /* eliminate trailing slash */
 	    p[len - 1] = '\0';
@@ -234,7 +139,7 @@ xlocaledir(
     if (dir) {
 #ifndef WIN32
 	/*
-	 * Only use the user-supplied path if the process isn't priviledged.
+	 * Only use the user-supplied path if the process isn't privileged.
 	 */
 	if (getuid() == geteuid() && getgid() == getegid()) {
 #if defined(HASSETUGID)
@@ -249,14 +154,14 @@ xlocaledir(
 	    }
 #else
 	    /*
-	     * If there are saved ID's the process might still be priviledged
+	     * If there are saved ID's the process might still be privileged
 	     * even though the above test succeeded.  If issetugid() and
 	     * getresgid() aren't available, test this by trying to set
 	     * euid to 0.
 	     *
 	     * Note: this only protects setuid-root clients.  It doesn't
 	     * protect other setuid or any setgid clients.  If this tradeoff
-	     * isn't acceptable, set DisableXLocaleDirEnv to YES in host.def.
+	     * isn't acceptable, run configure with --disable-xlocaledir .
 	     */
 	    unsigned int oldeuid;
 	    oldeuid = geteuid();
@@ -264,7 +169,7 @@ xlocaledir(
 		priv = 0;
 	    } else {
 		if (seteuid(oldeuid) == -1) {
-		    /* XXX ouch, coudn't get back to original uid
+		    /* XXX ouch, couldn't get back to original uid
 		     what can we do ??? */
 		    _exit(127);
 		}
@@ -276,8 +181,8 @@ xlocaledir(
 	priv = 0;
 #endif
 	if (!priv) {
-	    len = strlen(dir);
-	    strncpy(p, dir, buf_len);
+	    len = (int) strlen(dir);
+	    strncpy(p, dir, (size_t) buf_len);
 	    if (len < buf_len) {
 	        p[len++] = LC_PATHDELIM;
 	        p += len;
@@ -287,11 +192,7 @@ xlocaledir(
 #endif /* NO_XLOCALEDIR */
 
     if (len < buf_len)
-#ifndef __UNIXOS2__
-      strncpy(p, XLOCALEDIR, buf_len - len);
-#else
-      strncpy(p,__XOS2RedirRoot(XLOCALEDIR), buf_len - len);
-#endif
+      strncpy(p, XLOCALEDIR, (size_t) (buf_len - len));
     buf[buf_len-1] = '\0';
 }
 
@@ -312,7 +213,7 @@ xlocalelibdir(
     if (dir) {
 #ifndef WIN32
 	/*
-	 * Only use the user-supplied path if the process isn't priviledged.
+	 * Only use the user-supplied path if the process isn't privileged.
 	 */
 	if (getuid() == geteuid() && getgid() == getegid()) {
 #if defined(HASSETUGID)
@@ -327,14 +228,14 @@ xlocalelibdir(
 	    }
 #else
 	    /*
-	     * If there are saved ID's the process might still be priviledged
+	     * If there are saved ID's the process might still be privileged
 	     * even though the above test succeeded.  If issetugid() and
 	     * getresgid() aren't available, test this by trying to set
 	     * euid to 0.
 	     *
 	     * Note: this only protects setuid-root clients.  It doesn't
 	     * protect other setuid or any setgid clients.  If this tradeoff
-	     * isn't acceptable, set DisableXLocaleDirEnv to YES in host.def.
+	     * isn't acceptable, run configure with --disable-xlocaledir .
 	     */
 	    unsigned int oldeuid;
 	    oldeuid = geteuid();
@@ -342,7 +243,7 @@ xlocalelibdir(
 		priv = 0;
 	    } else {
 		if (seteuid(oldeuid) == -1) {
-		    /* XXX ouch, coudn't get back to original uid
+		    /* XXX ouch, couldn't get back to original uid
 		     what can we do ??? */
 		    _exit(127);
 		}
@@ -354,8 +255,8 @@ xlocalelibdir(
 	priv = 0;
 #endif
 	if (!priv) {
-	    len = strlen(dir);
-	    strncpy(p, dir, buf_len);
+	    len = (int) strlen(dir);
+	    strncpy(p, dir, (size_t) buf_len);
 	    if (len < buf_len) {
 	        p[len++] = LC_PATHDELIM;
 	        p += len;
@@ -365,11 +266,7 @@ xlocalelibdir(
 #endif /* NO_XLOCALEDIR */
 
     if (len < buf_len)
-#ifndef __UNIXOS2__
-      strncpy(p, XLOCALELIBDIR, buf_len - len);
-#else
-      strncpy(p,__XOS2RedirRoot(XLOCALELIBDIR), buf_len - len);
-#endif
+      strncpy(p, XLOCALELIBDIR, (size_t) (buf_len - len));
     buf[buf_len-1] = '\0';
 }
 
@@ -396,17 +293,6 @@ resolve_name(
 	char *p = buf;
 	int n;
 	char *args[2], *from, *to;
-#ifdef __UNIXOS2__  /* Take out CR under OS/2 */
-	int len;
-
-	len = strlen(p);
-	if (len > 1) {
-	    if (*(p+len-2) == '\r' && *(p+len-1) == '\n') {
-		*(p+len-2) = '\n';
-		*(p+len-1) = '\0';
-	    }
-	}
-#endif
 	while (isspace(*p)) {
 	    ++p;
 	}
@@ -442,7 +328,7 @@ lowercase(
     char *t;
 
     for (s = src, t = dst; *s; ++s, ++t)
-	*t = c_tolower(*s);
+	*t = (char) c_tolower(*s);
     *t = '\0';
     return dst;
 }
@@ -468,7 +354,7 @@ normalize_lcname (const char *name)
 	    *p++ = *tmp++;
 	while (*tmp) {
 	    if (*tmp != '-')
-		*p++ = c_tolower(*tmp);
+		*p++ = (char) c_tolower(*tmp);
 	    tmp++;
 	}
     }
@@ -508,9 +394,11 @@ _XlcFileName(
     for (i = 0; i < n; ++i) {
 	char buf[PATH_MAX], *name;
 
+	if (args[i] == NULL)
+	    continue;
+
 	name = NULL;
-	if ((5 + (args[i] ? strlen (args[i]) : 0) + strlen(cat)) < PATH_MAX) {
-	    sprintf(buf, "%s/%s.dir", args[i], cat);
+	if (snprintf(buf, PATH_MAX, "%s/%s.dir", args[i], cat) < PATH_MAX) {
 	    name = resolve_name(siname, buf, RtoL);
 	}
 	if (name == NULL) {
@@ -520,13 +408,13 @@ _XlcFileName(
 	    /* supposed to be absolute path name */
 	    file_name = name;
 	} else {
-	    file_name = Xmalloc(2 + (args[i] ? strlen (args[i]) : 0) +
-				(name ? strlen (name) : 0));
-	    if (file_name != NULL)
-		sprintf(file_name, "%s/%s", args[i], name);
+	    if (snprintf(buf, PATH_MAX, "%s/%s", args[i], name) < PATH_MAX)
+		file_name = strdup(buf);
+	    else
+		file_name = NULL;
 	    Xfree(name);
 	}
-	if (isreadable(file_name)) {
+	if (file_name && isreadable(file_name)) {
 	    break;
 	}
 	Xfree(file_name);
@@ -557,9 +445,11 @@ _XlcResolveLocaleName(
     xlocaledir (dir, PATH_MAX);
     n = _XlcParsePath(dir, args, NUM_LOCALEDIR);
     for (i = 0; i < n; ++i) {
-	if ((2 + (args[i] ? strlen (args[i]) : 0) +
-	    strlen (locale_alias)) < PATH_MAX) {
-	    sprintf (buf, "%s/%s", args[i], locale_alias);
+	if (args[i] == NULL)
+	    continue;
+
+	if (snprintf (buf, PATH_MAX, "%s/%s", args[i], locale_alias)
+	    < PATH_MAX) {
 	    name = resolve_name (lc_name, buf, LtoR);
 	    if (!name) {
 		if (!nlc_name)
@@ -572,7 +462,7 @@ _XlcResolveLocaleName(
 	    break;
 	}
     }
-    if (nlc_name) Xfree(nlc_name);
+    Xfree(nlc_name);
 
     if (name == NULL) {
 	/* vendor locale name == Xlocale name, no expansion of alias */
@@ -581,7 +471,7 @@ _XlcResolveLocaleName(
 	pub->siname = name;
     }
 
-    sinamelen = strlen (pub->siname);
+    sinamelen = (int) strlen (pub->siname);
     if (sinamelen == 1 && pub->siname[0] == 'C') {
 	pub->language = pub->siname;
 	pub->territory = pub->codeset = NULL;
@@ -633,14 +523,14 @@ _XlcResolveI18NPath(char *buf, int buf_len)
 }
 
 char *
-_XlcLocaleDirName(char *dir_name, size_t dir_len, char *lc_name)
+_XlcLocaleDirName(char *dir_name, size_t dir_len, const char *lc_name)
 {
-    char dir[PATH_MAX], buf[PATH_MAX], *name = NULL;
+    char dir[PATH_MAX], buf[PATH_MAX];
     int i, n;
     char *args[NUM_LOCALEDIR];
     static char locale_alias[] = LOCALE_ALIAS;
-    char *target_name = (char*)0;
-    char *target_dir = (char*)0;
+    char *target_name = NULL;
+    char *target_dir = NULL;
     char *nlc_name = NULL;
     static char*  last_dir_name = 0;
     static size_t last_dir_len = 0;
@@ -653,12 +543,15 @@ _XlcLocaleDirName(char *dir_name, size_t dir_len, char *lc_name)
     }
 
     xlocaledir (dir, PATH_MAX);
-    n = _XlcParsePath(dir, args, 256);
+    n = _XlcParsePath(dir, args, NUM_LOCALEDIR);
     for (i = 0; i < n; ++i) {
+	char *name = NULL;
 
-	if ((2 + (args[i] ? strlen(args[i]) : 0) +
- 	     strlen(locale_alias)) < PATH_MAX) {
- 	    sprintf (buf, "%s/%s", args[i], locale_alias);
+	if (args[i] == NULL)
+	    continue;
+
+	if (snprintf (buf, PATH_MAX, "%s/%s", args[i], locale_alias)
+	    < PATH_MAX) {
  	    name = resolve_name(lc_name, buf, LtoR);
 	    if (!name) {
 		if (!nlc_name)
@@ -668,25 +561,15 @@ _XlcLocaleDirName(char *dir_name, size_t dir_len, char *lc_name)
 	    }
  	}
 
- 	/* If name is not an alias, use lc_name for locale.dir search */
- 	if (name == NULL)
- 	    name = lc_name;
-
  	/* look at locale.dir */
 
  	target_dir = args[i];
- 	if (!target_dir) {
- 	    /* something wrong */
- 	    if (name != lc_name)
- 		Xfree(name);
- 	    continue;
+	if (snprintf(buf, PATH_MAX, "%s/locale.dir", target_dir) < PATH_MAX) {
+	    /* If name is not an alias, use lc_name for locale.dir search */
+	    target_name = resolve_name(name ? name : lc_name, buf, RtoL);
  	}
- 	if ((1 + strlen (target_dir) + strlen("locale.dir")) < PATH_MAX) {
- 	    sprintf(buf, "%s/locale.dir", target_dir);
- 	    target_name = resolve_name(name, buf, RtoL);
- 	}
- 	if (name != lc_name)
- 	    Xfree(name);
+	Xfree(name);
+	name = NULL;
  	if (target_name != NULL) {
  	    char *p = 0;
  	    if ((p = strstr(target_name, "/XLC_LOCALE"))) {
@@ -696,32 +579,19 @@ _XlcLocaleDirName(char *dir_name, size_t dir_len, char *lc_name)
  	    Xfree(target_name);
  	    target_name = NULL;
  	}
- 	name = NULL;
     }
-    if (nlc_name) Xfree(nlc_name);
+    Xfree(nlc_name);
 
-    if (target_name == NULL) {
+    if (target_name == NULL)
  	/* vendor locale name == Xlocale name, no expansion of alias */
- 	target_dir = args[0];
- 	target_name = lc_name;
-    }
-    /* snprintf(dir_name, dir_len, "%s/%", target_dir, target_name); */
-    strncpy(dir_name, target_dir, dir_len - 1);
-    if (strlen(target_dir) >= dir_len - 1) {
-	dir_name[dir_len - 1] = '\0';
-    } else  {
-	strcat(dir_name, "/");
-	strncat(dir_name, target_name, dir_len - strlen(dir_name) - 1);
-	if (strlen(target_name) >= dir_len - strlen(dir_name) - 1)
-	    dir_name[dir_len - 1] = '\0';
-    }
-    if (target_name != lc_name)
- 	Xfree(target_name);
+	snprintf(dir_name, dir_len, "%s/%s", args[0], lc_name);
+    else
+	snprintf(dir_name, dir_len, "%s/%s", target_dir, target_name);
+
+    Xfree(target_name);
+    Xfree (last_dir_name);
+    Xfree (last_lc_name);
 
-    if (last_dir_name != 0)
-	Xfree (last_dir_name);
-    if (last_lc_name != 0)
-	Xfree (last_lc_name);
     last_dir_len = strlen (dir_name) + 1;
     last_dir_name = Xmalloc (last_dir_len);
     strcpy (last_dir_name, dir_name);
@@ -731,14 +601,14 @@ _XlcLocaleDirName(char *dir_name, size_t dir_len, char *lc_name)
 }
 
 char *
-_XlcLocaleLibDirName(char *dir_name, size_t dir_len, char *lc_name)
+_XlcLocaleLibDirName(char *dir_name, size_t dir_len, const char *lc_name)
 {
-    char dir[PATH_MAX], buf[PATH_MAX], *name = NULL;
+    char dir[PATH_MAX], buf[PATH_MAX];
     int i, n;
     char *args[NUM_LOCALEDIR];
     static char locale_alias[] = LOCALE_ALIAS;
-    char *target_name = (char*)0;
-    char *target_dir = (char*)0;
+    char *target_name = NULL;
+    char *target_dir = NULL;
     char *nlc_name = NULL;
     static char*  last_dir_name = 0;
     static size_t last_dir_len = 0;
@@ -751,12 +621,15 @@ _XlcLocaleLibDirName(char *dir_name, size_t dir_len, char *lc_name)
     }
 
     xlocalelibdir (dir, PATH_MAX);
-    n = _XlcParsePath(dir, args, 256);
+    n = _XlcParsePath(dir, args, NUM_LOCALEDIR);
     for (i = 0; i < n; ++i) {
+	char *name = NULL;
 
-	if ((2 + (args[i] ? strlen(args[i]) : 0) +
- 	     strlen(locale_alias)) < PATH_MAX) {
- 	    sprintf (buf, "%s/%s", args[i], locale_alias);
+	if (args[i] == NULL)
+	    continue;
+
+	if (snprintf (buf, PATH_MAX, "%s/%s", args[i], locale_alias)
+	    < PATH_MAX) {
  	    name = resolve_name(lc_name, buf, LtoR);
 	    if (!name) {
 		if (!nlc_name)
@@ -766,25 +639,15 @@ _XlcLocaleLibDirName(char *dir_name, size_t dir_len, char *lc_name)
 	    }
  	}
 
- 	/* If name is not an alias, use lc_name for locale.dir search */
- 	if (name == NULL)
- 	    name = lc_name;
-
  	/* look at locale.dir */
 
  	target_dir = args[i];
- 	if (!target_dir) {
- 	    /* something wrong */
- 	    if (name != lc_name)
- 		Xfree(name);
- 	    continue;
+	if (snprintf(buf, PATH_MAX, "%s/locale.dir", target_dir) < PATH_MAX) {
+	    /* If name is not an alias, use lc_name for locale.dir search */
+	    target_name = resolve_name(name ? name : lc_name, buf, RtoL);
  	}
- 	if ((1 + strlen (target_dir) + strlen("locale.dir")) < PATH_MAX) {
- 	    sprintf(buf, "%s/locale.dir", target_dir);
- 	    target_name = resolve_name(name, buf, RtoL);
- 	}
- 	if (name != lc_name)
- 	    Xfree(name);
+	Xfree(name);
+	name = NULL;
  	if (target_name != NULL) {
  	    char *p = 0;
  	    if ((p = strstr(target_name, "/XLC_LOCALE"))) {
@@ -794,32 +657,18 @@ _XlcLocaleLibDirName(char *dir_name, size_t dir_len, char *lc_name)
  	    Xfree(target_name);
  	    target_name = NULL;
  	}
- 	name = NULL;
     }
-    if (nlc_name) Xfree(nlc_name);
+    Xfree(nlc_name);
 
-    if (target_name == NULL) {
+    if (target_name == NULL)
  	/* vendor locale name == Xlocale name, no expansion of alias */
- 	target_dir = args[0];
- 	target_name = lc_name;
-    }
-    /* snprintf(dir_name, dir_len, "%s/%", target_dir, target_name); */
-    strncpy(dir_name, target_dir, dir_len - 1);
-    if (strlen(target_dir) >= dir_len - 1) {
-	dir_name[dir_len - 1] = '\0';
-    } else  {
-	strcat(dir_name, "/");
-	strncat(dir_name, target_name, dir_len - strlen(dir_name) - 1);
-	if (strlen(target_name) >= dir_len - strlen(dir_name) - 1)
-	    dir_name[dir_len - 1] = '\0';
-    }
-    if (target_name != lc_name)
- 	Xfree(target_name);
+	snprintf(dir_name, dir_len, "%s/%s", args[0], lc_name);
+    else
+	snprintf(dir_name, dir_len, "%s/%s", target_dir, target_name);
+    Xfree(target_name);
+    Xfree (last_dir_name);
+    Xfree (last_lc_name);
 
-    if (last_dir_name != 0)
-	Xfree (last_dir_name);
-    if (last_lc_name != 0)
-	Xfree (last_lc_name);
     last_dir_len = strlen (dir_name) + 1;
     last_dir_name = Xmalloc (last_dir_len);
     strcpy (last_dir_name, dir_name);
